#    YADSync
#    Copyright (C) 2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import sys
import os.path
import os
import datetime
import gettext

import configfile
import tools


_=gettext.gettext

gettext.bindtextdomain( 'yadsync', '/usr/share/locale' )
gettext.textdomain( 'yadsync' )


class Config( configfile.ConfigFileWithProfiles ):
	APP_NAME = 'YADSync'
	VERSION = '1.1beta1'
	COPYRIGHT = 'Copyright (c) 2009 Oprea Dan'

	def __init__( self ):
		configfile.ConfigFileWithProfiles.__init__( self, _('Main profile') )

		self._APP_PATH = os.path.dirname( os.path.abspath( os.path.dirname( __file__ ) ) )
		self._DOC_PATH = '/usr/share/doc/yadsync'
		if os.path.exists( os.path.join( self._APP_PATH, 'LICENSE' ) ):
			self._DOC_PATH = self._APP_PATH

		HOME_FOLDER = os.path.expanduser( '~' )
		
		DEFAULT_CONFIG_PATH = '$HOME/.config'
		if sys.platform == 'win32':
			DEFAULT_CONFIG_PATH = '$HOME\\.config'

		self._LOCAL_CONFIG_FOLDER = os.path.join( os.getenv( 'XDG_CONFIG_HOME', DEFAULT_CONFIG_PATH ).replace( '$HOME', HOME_FOLDER ), 'yadsync' )
		tools.make_dirs( self._LOCAL_CONFIG_FOLDER )

		self._LOCAL_CONFIG_PATH = os.path.join( self._LOCAL_CONFIG_FOLDER, 'config' )

		self.load( self._LOCAL_CONFIG_PATH )

	def save( self ):
		configfile.ConfigFile.save( self, self._LOCAL_CONFIG_PATH )

	def can_sync( self ):
		if len( self.get_directories() ) <= 0:
			return False

		return True

	def get_rsync( self ):
		default_value = 'rsync'
		if sys.platform == 'win32':
			default_value = '"C:\\Program Files\\cwRsync\\bin\\rsync"'
		return self.get_str_value( 'sync.tools.rsync', default_value )

	def get_directories( self, profile_id = None ):
		size = self.get_profile_int_value( 'dir.size', profile_id = profile_id )
		items = []

		for i in xrange( 1, size + 1 ):
			src_path = self.get_profile_str_value( "dir.%s.src" % i, profile_id = profile_id )
			dest_path = self.get_profile_str_value( "dir.%s.dest" % i, profile_id = profile_id ) 
			items.append( ( src_path, dest_path ) )

		return items

	def set_directories( self, items ):
		self.remove_profile_keys_starts_with( 'dir.' )

		size = len( items )
		self.set_profile_int_value( 'dir.size', size )

		for i in xrange( 1, size + 1 ):
			self.set_profile_str_value( "dir.%s.src" % i, items[i-1][0] )
			self.set_profile_str_value( "dir.%s.dest" % i, items[i-1][1] )

	def get_exclude( self ):
		if self.has_profile_value( 'exclude.size' ):
			size = self.get_profile_int_value( 'exclude.size', 0 )
			items = []

			for i in xrange( 1, size + 1 ):
				items.append( self.get_profile_str_value( "exclude.%s.value" % i ) )

			return items
		
		return [ '.*', '*~', '*.swp' ]

	def set_exclude( self, items ):
		self.remove_profile_keys_starts_with( 'exclude.' )

		size = len( items )
		self.set_profile_int_value( 'exclude.size', size )

		for i in xrange( 1, size + 1 ):
			self.set_profile_str_value( "exclude.%s.value" % i, items[i-1] )

	def get_enable_delete( self ):
		return self.get_profile_bool_value( 'options.delete', False )

	def set_enable_delete( self, value ):
		self.set_profile_bool_value( 'options.delete', value )

	def get_enable_backup( self ):
		return self.get_profile_bool_value( 'options.backup', True )

	def set_enable_backup( self, value ):
		self.set_profile_bool_value( 'options.backup', value )

	def get_app_path( self ):
		return self._APP_PATH

	def get_doc_path( self ):
		return self._DOC_PATH

	def get_license( self ):
		return tools.read_file( os.path.join( self.get_doc_path(), 'LICENSE' ), '' )

	def get_translations( self ):
		return tools.read_file( os.path.join( self.get_doc_path(), 'TRANSLATIONS' ), '' )

	def get_authors( self ):
		return tools.read_file( os.path.join( self.get_doc_path(), 'AUTHORS' ), '' )

