#    YADSync
#    Copyright (C) 2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import subprocess
import sys
import os
import os.path


def read_file( path, default_value = None ):
	ret_val = default_value 

	try:
		file = open( path )
		ret_val = file.read()
		file.close()
	except:
		pass

	return ret_val


def read_file_lines( path, default_value = None ):
	ret_val = default_value 

	try:
		file = open( path )
		ret_val = file.readlines()
		file.close()
	except:
		pass

	return ret_val


def read_command_output( cmd ):
	ret_val = ''

	try:
		use_shell = True
		if sys.platform == 'win32':
			use_shell = False

		process = subprocess.Popen( cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=use_shell )
		pipe = process.stdout
		ret_val = pipe.read().strip()
		pipe.close() 
	except:
		return ''

	return ret_val


def read_command_output_with_callback( cmd, callback ):
	ret_val = -1

	try:
		use_shell = True
		if sys.platform == 'win32':
			use_shell = False

		process = subprocess.Popen( cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=use_shell )
		pipe = process.stdout
		
		for line in pipe.xreadlines():
			line = line.rstrip( '\n\r' )
			callback( line )

		pipe.close()
		ret_val = process.wait()
	except:
		pass

	if ret_val is None:
		return 0

	return ret_val


def check_command( cmd ):
	cmd = cmd.strip()

	if len( cmd ) < 1:
		return False

	if os.path.isfile( cmd ):
		return True

	cmd = read_command_output( "which \"%s\"" % cmd )

	if len( cmd ) < 1:
		return False

	if os.path.isfile( cmd ):
		return True

	return False


def prepare_path( path ):
	if sys.platform == 'win32':
		path = path.strip( "\\" )
	else:
		path = path.strip( "/" )
		path = os.sep + path
	return path


def make_dirs( path ):
	if not os.path.isdir( path ):
		try:
			os.makedirs( path )
		except:
			pass

def is_local_path( path ):
	index = path.find( ':' )
	if index < 0:
		return True

	if sys.platform == 'win32':
		if index <= 1:
			return True

	return False

def prepare_rsync_path( path ):
	if sys.platform == 'win32':
		if is_local_path( path ):
			path = path.replace( '\\', '/' )
			drive = path[0].lower()
			path = '/cygdrive/' + drive + path[ 2 : ]

	return path


