#    YADSync
#    Copyright (C) 2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import os.path
import sys
import datetime
import time
import gettext

import config
import tools
import driveinfo


_=gettext.gettext


class YADSync:
	def __init__( self, config, callback = None ):
		self.config = config
		self.callback = callback

	def get_callback( self ):
		return self.callback

	def set_callback( self, callback ):
		self.callback = callback

	def get_config( self ):
		return self.config

	def set_config( self, config ):
		self.config = config

	def notify( self, type, message ):
		if not self.callback is None:
			self.callback( type, message )

	def __execute_callback__( self, line ):
		self.notify( 'cmd.output', line )

	def execute( self, cmd ):
		self.notify( 'cmd', cmd )
		ret_val = tools.read_command_output_with_callback( cmd, self.__execute_callback__ )
		if 0 == ret_val:
			self.notify( 'cmd.ok', _('Returns: %s') % ret_val )
		else:
			self.notify( 'cmd.warning', _('Returns: %s') % ret_val )
		return ret_val

	def save( self ):
		self.sync( False, True )

	def restore( self ):
		self.sync( True, False )

	def sync( self, update_src = True, update_dest = True ): 
		if not ( update_src or update_dest ): #nothing to do
			return

		name = _('Sync')
		symb = '<=>'
		enable_delete = False
		if not update_src and update_dest:
			name = _('Save')
			symb = '=>'
			enable_delete = True
		elif update_src and not update_dest:
			name = _('Restore')
			symb = '<='
			enable_delete = True
		
		directories = self.config.get_directories()
		exclude = self.config.get_exclude()

		#rsync
		date_str = datetime.datetime.today().strftime( '%Y-%m-%d %Hh%Mm%Ss' )
		rsync = "%s -avbu " % self.config.get_rsync()

		if self.config.get_enable_backup():
			rsync = rsync + "--backup-dir=\"%s\" " % os.path.join( '.backup', date_str )

		if enable_delete and self.config.get_enable_delete():
			rsync = rsync + '--delete '

		#exclude
		rsync = rsync + '--exclude=".backup" '

		for item in exclude:
			rsync = rsync + "--exclude=\"%s\" " % item

		check_log = False

		for dir in directories:
			src = dir[0]
			dest = dir[1]

			self.notify( 'title', "%s: %s %s %s" % ( name, src, symb, dest ) )

			if not os.path.isdir( src ):
				self.notify( 'warning', _('Source don\'t exists: %s !') % src )
				check_log = True
				continue

			if not os.path.isdir( dest ):
				self.notify( 'warning', _('Destination don\'t exists: %s !') % dest )
				check_log = True
				continue

			src_backup_path = os.path.join( src, '.backup' )
			tools.make_dirs( src_backup_path )
			src_driveinfo = driveinfo.DriveInfo( src_backup_path )
			rsync_src_extra = ' '
			if sys.platform != 'win32':
				if not src_driveinfo.support_usergroup():
					rsync_src_extra = rsync_src_extra + ' --no-owner --no-group '
				if not src_driveinfo.support_permissions():
					rsync_src_extra = rsync_src_extra + ' --no-perms --chmod=Dog=rx,Du=rwx,Fog=r,Fu=rw '

			dest_backup_path = os.path.join( dest, '.backup' )
			tools.make_dirs( dest_backup_path )
			dest_driveinfo = driveinfo.DriveInfo( dest_backup_path )
			rsync_dest_extra = ''
			if sys.platform != 'win32':
				if not dest_driveinfo.support_usergroup():
					rsync_dest_extra = rsync_dest_extra + ' --no-owner --no-group '
				if not dest_driveinfo.support_permissions():
					rsync_dest_extra = rsync_dest_extra + ' --no-perms --chmod=Dog=rx,Du=rwx,Fog=r,Fu=rw '

			rsync_src = tools.prepare_rsync_path( src )
			rsync_dest = tools.prepare_rsync_path( dest )


			if update_dest:
				if 0 != self.execute( "%s %s \"%s/./\" \"%s\"" % ( rsync, rsync_src_extra, rsync_src, rsync_dest ) ):
					check_log = True

			if update_src:
				if 0 != self.execute( "%s %s \"%s/./\" \"%s\"" % ( rsync, rsync_dest_extra, rsync_dest, rsync_src ) ):
					check_log = True

		if check_log:
			self.notify( 'error', _('It seems there was some errors or warnings ! You should check the log !') )
		else:
			self.notify( 'title', _('All OK !') )


def print_help( cfg, app_name ):
	print 'Options:'
	print '--version\n\tShow version'
	print '--help\n\tShow this help'
	print '--license\n\tShow license'
	#print '--profile <profile name>\n\tSelect the profile'
	#print '--sync\n\tRun "sync" process'
	#print '--save\n\tRun "save" process'
	#print '--restore\n\tRun "restore" process'


def app( app_name = 'yadsync' ):
	cfg = config.Config()

	print 'YADSync'
	print 'Version: ' + cfg.VERSION
	print ''
	print 'YADSync comes with ABSOLUTELY NO WARRANTY.'
	print 'This is free software, and you are welcome to redistribute it'
	print "under certain conditions; type `%s --license\' for details." % app_name
	print ''

	for arg in sys.argv[ 1 : ]:
		if arg == '--version' or arg == '-v':
			sys.exit(0)

		if arg == '--license':
			print cfg.get_license()
			sys.exit(0)

		if arg == '--help' or arg == '-h':
			print_help( cfg, app_name )
			sys.exit(0)

	return cfg


if __name__ == '__main__':
	app()

