#    YADSync
#    Copyright (C) 2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import os.path
import stat
import sys

import pygtk
pygtk.require("2.0")
import gtk
import gobject
import gtk.glade
import pango
import datetime
import gettext
import time

sys.path = [os.path.join( os.path.dirname( os.path.abspath( os.path.dirname( __file__ ) ) ), 'common' )] + sys.path

import config
import tools
import yadsync

import messagebox
from settingsdialog import *

_=gettext.gettext


class AboutDialog( gtk.AboutDialog ):
	def __init__( self, config, parent ):
		gtk.AboutDialog.__init__( self )
	
		self.connect( 'close', self.on_close )
		self.connect( 'response', self.on_close )

		self.set_transient_for( parent )

		self.set_name( config.APP_NAME )
		self.set_version( config.VERSION )
		self.set_copyright( 'Copyright (C) 2009 Oprea Dan' )
		self.set_website( 'http://yadsync.le-web.org' )
		self.set_website_label( 'http://yadsync.le-web.org' )
		self.set_license( config.get_license() )
		authors = config.get_authors()
		if not authors is None:
			self.set_authors( authors.split( '\n' ) )

		extra_translations = _('about-translators').strip()
		if extra_translations == 'about-translators':
			extra_translations = ''

		self.set_translator_credits( config.get_translations() + extra_translations  )

	def on_close( self, *params ):
		self.destroy()


class MainWindow:
	def __init__( self, config ):
		self.config = config

		self.glade = gtk.glade.XML( os.path.join( self.config.get_app_path(), 'gnome', 'mainwindow.glade' ), None, 'yadsync' )

		signals = { 
				'on_MainWindow_destroy' : gtk.main_quit,
				'on_MainWindow_delete_event' : self.on_close,
				'on_btn_settings_clicked' : self.on_btn_settings_clicked,
				'on_btn_about_clicked' : self.on_btn_about_clicked,
				'on_btn_help_clicked' : self.on_btn_help_clicked,
				'on_btn_exit_clicked' : self.on_close,
				'on_btn_sync_clicked' : self.on_btn_sync_clicked,
				'on_btn_save_clicked' : self.on_btn_save_clicked,
				'on_btn_restore_clicked' : self.on_btn_restore_clicked,
				'on_combo_profiles_changed' : self.on_combo_profiles_changed,
			}

		self.glade.signal_autoconnect( signals )

		self.window = self.glade.get_widget( 'MainWindow' )
		self.window.set_title( self.config.APP_NAME )

		#txt log
		self.scroll_txt_log = self.glade.get_widget( 'scroll_txt_log' )
		self.txt_log = self.glade.get_widget( 'txt_log' )
		self.txt_buffer = self.txt_log.get_buffer()
	
		self.txt_buffer.create_tag( 'text' )
		self.txt_buffer.create_tag( 'title', weight=700 )
		self.txt_buffer.create_tag( 'cmd', family='Monospace', weight=700 )
		self.txt_buffer.create_tag( 'cmd.output', family='Monospace' )
		self.txt_buffer.create_tag( 'cmd.warning', family='Monospace', foreground='blue', weight=700 )
		self.txt_buffer.create_tag( 'cmd.error', family='Monospace', foreground='red', weight=700 )
		self.txt_buffer.create_tag( 'cmd.ok', family='Monospace', weight=700 )
		self.txt_buffer.create_tag( 'warning', foreground='blue', weight=700 )
		self.txt_buffer.create_tag( 'error', foreground='red', weight=700 )

		#profiles ( name, id )
		self.store_profiles = gtk.ListStore( str, str )

		self.combo_profiles = self.glade.get_widget( 'combo_profiles' )
		self.combo_profiles.set_model( self.store_profiles )

		text_renderer = gtk.CellRendererText()
		self.combo_profiles.pack_start( text_renderer, True )
		self.combo_profiles.add_attribute( text_renderer, 'markup', 0 )

		#update sync buttons
		self.update_profile( True )

		#show main window
		self.window.show()

	def on_combo_profiles_changed( self, *params ):
		iter = self.combo_profiles.get_active_iter()
		if iter is None:
			return

		profile_id = self.store_profiles.get_value( iter, 1 )
		self.config.set_current_profile( profile_id )
		self.update_profile()

	def update_profile( self, populate_profiles = False ):
		if populate_profiles:
			self.combo_profiles.get_model().clear()
			
			profiles = self.config.get_profiles_sorted_by_name()

			for profile_id in profiles:
				iter = self.store_profiles.append( [ self.config.get_profile_name( profile_id ), profile_id ] )
				if profile_id == self.config.get_current_profile():
					self.combo_profiles.set_active_iter( iter )

		enabled = self.config.can_sync()
		self.glade.get_widget( 'btn_sync' ).set_sensitive( enabled )
		self.glade.get_widget( 'btn_save' ).set_sensitive( enabled )
		self.glade.get_widget( 'btn_restore' ).set_sensitive( enabled )

	def on_btn_settings_clicked( self, button ):
		dlg = SettingsDialog( self.config, self.window )
		dlg.run()
		self.update_profile( True )

	def on_btn_about_clicked( self, button ):
		dlg = AboutDialog( self.config, self.window )
		dlg.run()

	def on_btn_help_clicked( self, button ):
		messagebox.show_error( self.window, self.config, _('Not implemented') )

	def on_close( self, *params ):
		self.window.destroy()
		return True

	def on_btn_sync_clicked( self, button ):
		self.sync( True, True )

	def on_btn_save_clicked( self, button ):
		self.sync( False, True )

	def on_btn_restore_clicked( self, button ):
		self.sync( True, False )

	def log_clear( self ):
		self.txt_buffer.set_text( '' )
		self._update_loop_()

	def log_append( self, text, tag = None ):
		if tag is None or len(tag) <= 0:
			self.txt_buffer.insert( self.txt_buffer.get_end_iter(), text + '\n' )
		else:
			self.txt_buffer.insert_with_tags_by_name( self.txt_buffer.get_end_iter(), text + '\n', tag )

		end_iter = self.txt_buffer.get_end_iter()
		self.txt_buffer.place_cursor( end_iter )
		self.txt_log.scroll_to_iter( end_iter, 0.0 )

		self._update_loop_()

	def _update_loop_( self ):
		gtk.gdk.window_process_all_updates()
		
		while gtk.events_pending():
			gtk.main_iteration( False )

	def __sync_callback__( self, type, message ):
		self.log_append( message, type )

		if type in [ 'error', 'warning', 'title', 'cmd.ok', 'cmd.error', 'cmd.warning' ]:
			self.log_append( '' )
			
	def sync( self, update_src, update_dest ):
		self.window.set_sensitive( False )
		self.log_clear()
		self._update_loop_()

		yadsync.YADSync( self.config, self.__sync_callback__ ).sync( update_src, update_dest )

		self.window.set_sensitive( True )


def open_url( dialog, link, user_data ):
	os.system( "gnome-open \"%s\" &" % link )


def run():
	cfg = yadsync.app( 'yadsync-gnome' )
	gtk.about_dialog_set_url_hook( open_url, None )
	gtk.link_button_set_uri_hook( open_url, None )
	main_window = MainWindow( cfg )
	gtk.main()


if __name__ == '__main__':
	run()

