#    YADSync
#    Copyright (C) 2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import os.path

import pygtk
pygtk.require("2.0")
import gtk
import gobject
import gtk.glade
import datetime
import gettext
import time
import copy

import config
import tools
import messagebox


_=gettext.gettext


class SettingsDialog:
	def __init__( self, config, parent ):
		self.config = config
		self.config_copy_dict = copy.copy( self.config.dict )
		self.current_profile_org = self.config.get_current_profile()

		self.glade = gtk.glade.XML( os.path.join( self.config.get_app_path(), 'gnome', 'settingsdialog.glade' ), None, 'yadsync' )

		signals = { 
				#'on_SettingsDialog_response' : self.on_response,
				'on_btn_src_dir_clicked' : self.on_btn_src_dir_clicked,
				'on_btn_dest_dir_clicked' : self.on_btn_dest_dir_clicked,
				'on_btn_add_dir_clicked' : self.on_btn_add_dir_clicked,
				'on_btn_remove_dir_clicked' : self.remove_dir,
				'on_btn_add_exclude_clicked' : self.on_btn_add_exclude_clicked,
				'on_btn_remove_exclude_clicked' : self.remove_exclude,
				'on_combo_profiles_changed' : self.on_combo_profiles_changed,
				'on_btn_add_profile_clicked' : self.on_btn_add_profile_clicked,
				'on_btn_rename_profile_clicked' : self.on_btn_rename_profile_clicked,
				'on_btn_delete_profile_clicked' : self.on_btn_delete_profile_clicked,
			}

		self.dialog = self.glade.get_widget( 'SettingsDialog' )
		self.dialog.set_transient_for( parent )

		#profiles ( name, id )
		self.store_profiles = gtk.ListStore( str, str )

		self.combo_profiles = self.glade.get_widget( 'combo_profiles' )
		self.combo_profiles.set_model( self.store_profiles )

		text_renderer = gtk.CellRendererText()
		self.combo_profiles.pack_start( text_renderer, True )
		self.combo_profiles.add_attribute( text_renderer, 'markup', 0 )

		#directories
		self.store_dir = gtk.ListStore( str, str, str, str ) #icon source, source, icon dest, dest
		self.list_dir = self.glade.get_widget( 'list_dir' )
		self.list_dir.set_model( self.store_dir )

		pix_renderer = gtk.CellRendererPixbuf()
		text_renderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn( _('Source') )
		column.pack_start( pix_renderer, False )
		column.pack_end( text_renderer, True )
		column.add_attribute( pix_renderer, 'icon-name', 0 )
		column.add_attribute( text_renderer, 'text', 1 )
		column.set_sizing( gtk.TREE_VIEW_COLUMN_AUTOSIZE )
		column.set_expand(True)
		self.list_dir.append_column( column )

		pix_renderer = gtk.CellRendererPixbuf()
		text_renderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn( _('Destination') )
		column.pack_start( pix_renderer, False )
		column.pack_end( text_renderer, True )
		column.add_attribute( pix_renderer, 'icon-name', 2 )
		column.add_attribute( text_renderer, 'text', 3 )
		column.set_sizing( gtk.TREE_VIEW_COLUMN_AUTOSIZE )
		column.set_expand(True)
		self.list_dir.append_column( column )

		#src & dest
		self.edit_src = self.glade.get_widget( 'edit_src' )
		self.edit_dest = self.glade.get_widget( 'edit_dest' )

		#exclude
		self.store_exclude = gtk.ListStore( str, str ) #icon, value
		self.list_exclude = self.glade.get_widget( 'list_exclude' )
		self.list_exclude.set_model( self.store_exclude )

		pix_renderer = gtk.CellRendererPixbuf()
		text_renderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn( _('Exclude') )
		column.pack_start( pix_renderer, False )
		column.pack_end( text_renderer, True )
		column.add_attribute( pix_renderer, 'icon-name', 0 )
		column.add_attribute( text_renderer, 'text', 1 )
		self.list_exclude.append_column( column )

		#options
		self.cb_enable_backup = self.glade.get_widget( 'cb_enable_backup' )
		self.cb_enable_delete = self.glade.get_widget( 'cb_enable_delete' )

		self.update_profiles()
		self.glade.signal_autoconnect( signals )

	def on_btn_delete_profile_clicked( self, button ):
		if gtk.RESPONSE_YES == messagebox.show_question( self.dialog, self.config, _('Are you sure you want to delete the profile "%s" ?') % self.config.get_profile_name() ):
			self.config.remove_profile()
			self.update_profiles()

	def on_btn_rename_profile_clicked( self, button ):
		name = messagebox.text_input_dialog( self.dialog, self.config, _('Rename profile'), None, _('Name:') )
		if name is None:
			return
		if len( name ) <= 0:
			return

		if not self.config.set_profile_name( name ):
			return

		self.update_profiles()

	def on_btn_add_profile_clicked( self, button ):
		name = messagebox.text_input_dialog( self.dialog, self.config, _('New profile'), None, _('Name:') )
		if name is None:
			return
		if len( name ) <= 0:
			return

		if not self.config.add_profile( name ):
			return

		self.update_profiles()

	def on_combo_profiles_changed( self, *params ):
		iter = self.combo_profiles.get_active_iter()
		if iter is None:
			return

		profile_id = self.store_profiles.get_value( iter, 1 )

		if profile_id != self.config.get_current_profile():
			self.save_current_profile()
			self.config.set_current_profile( profile_id )
			self.update_profile()

	def update_profiles( self ):
		self.store_profiles.clear()

		profiles = self.config.get_profiles_sorted_by_name()
		
		for profile_id in profiles:
			iter = self.store_profiles.append( [ self.config.get_profile_name( profile_id ), profile_id ] )
			if profile_id == self.config.get_current_profile():
				self.combo_profiles.set_active_iter( iter )

		self.update_profile()

	def update_profile( self ):
		enabled = self.config.get_current_profile() != '1'
		self.glade.get_widget( 'btn_rename_profile' ).set_sensitive( enabled )
		self.glade.get_widget( 'btn_delete_profile' ).set_sensitive( enabled )

		#populate directories
		self.glade.get_widget( 'btn_remove_dir' ).set_sensitive( False )
		self.store_dir.clear()
		items = self.config.get_directories()
		for item in items:
			self.add_dir( item[0], item[1] )

		#populate exclude
		self.glade.get_widget( 'btn_remove_exclude' ).set_sensitive( False )
		self.store_exclude.clear()
		items = self.config.get_exclude()
		for item in items:
			self.add_exclude( item )

		#options
		self.cb_enable_backup.set_active( self.config.get_enable_backup() )
		self.cb_enable_delete.set_active( self.config.get_enable_delete() )

	def error_handler( self, message ):
		messagebox.show_error( self.dialog, self.config, message )

	def question_handler( self, message ):
		return gtk.RESPONSE_YES == messagebox.show_question( self.dialog, self.config, message )

	def add_dir( self, path, dest ):
		self.store_dir.append( [ 'gtk-directory', path, 'gtk-save', dest ] )

		if self.list_dir.get_selection().get_selected()[1] is None:
			self.list_dir.get_selection().select_iter( self.store_dir.get_iter_first() )

		self.glade.get_widget( 'btn_remove_dir' ).set_sensitive( True )

	def remove_dir( self, *params ):
		iter = self.list_dir.get_selection().get_selected()[1]
		if iter is None:
			return

		self.store_dir.remove( iter )

		if len( self.store_dir ) <= 0:
			self.glade.get_widget( 'btn_remove_dir' ).set_sensitive( False )
		else:
			self.list_dir.get_selection().select_iter( self.store_dir.get_iter_first() )

	def add_exclude( self, exclude ):
		self.store_exclude.append( [ 'gtk-stop', exclude ] )

		if self.list_exclude.get_selection().get_selected()[1] is None:
			self.list_exclude.get_selection().select_iter( self.store_exclude.get_iter_first() )

		self.glade.get_widget( 'btn_remove_exclude' ).set_sensitive( True )

	def remove_exclude( self, *params ):
		iter = self.list_exclude.get_selection().get_selected()[1]
		if iter is None:
			return

		self.store_exclude.remove( iter )

		if len( self.store_exclude ) <= 0:
			self.glade.get_widget( 'btn_remove_exclude' ).set_sensitive( False )
		else:
			self.list_exclude.get_selection().select_iter( self.store_exclude.get_iter_first() )

	def save_current_profile( self ):
		#directories
		items = []
		iter = self.store_dir.get_iter_first()
		while not iter is None:
			items.append( [ self.store_dir.get_value( iter, 1 ), self.store_dir.get_value( iter, 3 ) ] )
			iter = self.store_dir.iter_next( iter )
		self.config.set_directories( items )

		#exclude
		items = []
		iter = self.store_exclude.get_iter_first()
		while not iter is None:
			items.append( self.store_exclude.get_value( iter, 1 ) )
			iter = self.store_exclude.iter_next( iter )
		self.config.set_exclude( items )

		#options
		self.config.set_enable_backup( self.cb_enable_backup.get_active() )
		self.config.set_enable_delete( self.cb_enable_delete.get_active() )

	def save_config( self ):
		self.save_current_profile()

		#check
		profiles = self.config.get_profiles()
		for profile_id in profiles:
			if len( self.config.get_directories(profile_id) ) <= 0:
				messagebox.show_error( self.dialog, self.config, _('Profile: %s') % self.config.get_profile_name( profile_id ) + '\n' + _('There are no folders !') )
				return False

		#save
		self.config.save()
		return True

	#def on_response( self, dialog, response_id, *params ):
	#	if gtk.RESPONSE_OK == response_id:
	#		if not self.save_config():
	#			return
	#	else:
	#		self.config.dict = self.config_copy_dict

	#	self.config.set_current_profile( self.current_profile_org )
	#	
	#	self.dialog.destroy()

	def on_btn_src_dir_clicked( self, button ):
		path = self.edit_src.get_text()
		#if not tools.is_local_path( path ):
		#	path = ''

		fcd = gtk.FileChooserDialog( _('Source folder'), self.dialog, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK) )
		if len( path ) > 0:
			fcd.set_filename( path )

		if fcd.run() == gtk.RESPONSE_OK:
			path = tools.prepare_path( fcd.get_filename() )
			self.edit_src.set_text( path )

		fcd.destroy()

	def on_btn_dest_dir_clicked( self, button ):
		path = self.edit_dest.get_text()
		if not tools.is_local_path( path ):
			path = ''

		fcd = gtk.FileChooserDialog( _('Destination folder'), self.dialog, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK) )
		#fcd.set_local_only( False )
		if len( path ) > 0:
			fcd.set_filename( path )

		if fcd.run() == gtk.RESPONSE_OK:
			path = tools.prepare_path( fcd.get_filename() )
			self.edit_dest.set_text( path )

		fcd.destroy()

	def on_btn_add_dir_clicked( self, button ):
		src_path = self.edit_src.get_text().strip()
		if len( src_path ) <= 0:
			self.on_btn_src_dir_clicked( None )
			src_path = self.edit_src.get_text().strip()
			if len( src_path ) <= 0:
				return

		dest_path = self.edit_dest.get_text().strip()
		if len( dest_path ) <= 0:
			self.on_btn_dest_dir_clicked( None )
			dest_path = self.edit_dest.get_text().strip()
			if len( dest_path ) <= 0:
				return

		if src_path == dest_path:
			messagebox.show_error( self.dialog, self.config, _('Source and destination are the same !') )
			return

		if len( src_path ) <= 1:
			messagebox.show_error( self.dialog, self.config, _('Source can\'t be root !') )
			return

		if tools.is_local_path( src_path ):
			if not os.path.isdir( src_path ):
				messagebox.show_error( self.dialog, self.config, _('Source folder don\'t exists !') )
				return

		if len( dest_path ) <= 1:
			messagebox.show_error( self.dialog, self.config, _('Destination can\'t be root !') )
			return

		if tools.is_local_path( dest_path ):
			if not os.path.isdir( dest_path ):
				messagebox.show_error( self.dialog, self.config, _('Destination folder don\'t exists !') )
				return

		iter = self.store_dir.get_iter_first()
		while not iter is None:
			if self.store_dir.get_value( iter, 1 ) == src_path and  self.store_dir.get_value( iter, 3 ) == dest_path:
				messagebox.show_error( self.dialog, self.config, _('Source and destination already exists !') )
				return

			if self.store_dir.get_value( iter, 1 ) == src_path:
				if gtk.RESPONSE_YES != messagebox.show_question( self.dialog, self.config, _('Source folder already exists\n%s\nAre you sure you want to add it again ?') % src_path ):
					return

			if self.store_dir.get_value( iter, 3 ) == dest_path:
				if gtk.RESPONSE_YES != messagebox.show_question( self.dialog, self.config, _('Destination folder already exists:\n%s\nAre you sure you want to add it again ?') % dest_path ):
					return
			iter = self.store_dir.iter_next( iter )

		#add source <=> destination
		self.add_dir( src_path, dest_path )
		
		self.edit_src.set_text('')
		self.edit_dest.set_text('')

	def on_btn_add_exclude_clicked( self, button ):
		pattern = messagebox.text_input_dialog( self.dialog, self.config, title=_('Add exclude'), label=_('Pattern'), icon='gtk-add' )
		if pattern is None:
			return

		pattern = pattern.strip()
		if len( pattern ) <= 0:
			messagebox.show_error( self.dialog, self.config, _('Exclude pattern can\'t be empty') )
			return

		iter = self.store_exclude.get_iter_first()
		while not iter is None:
			if self.store_exclude.get_value( iter, 1 ) == pattern:
				messagebox.show_error( self.dialog, self.config, _('Pattern "%s" already exists !') % pattern )
				return
			iter = self.store_exclude.iter_next( iter )

		self.add_exclude( pattern )

	def run( self ):
		self.config.set_question_handler( self.question_handler )
		self.config.set_error_handler( self.error_handler )

		while True:
			response_id = self.dialog.run()

			if gtk.RESPONSE_OK == response_id:
				if not self.save_config():
					continue
			else:
				self.config.dict = self.config_copy_dict

			break

		self.config.set_current_profile( self.current_profile_org )
		self.dialog.destroy()
		self.config.clear_handlers()

