#    YADSync
#    Copyright (C) 2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import os.path
import stat
import sys
import datetime
import gettext
import time

sys.path = [os.path.join( os.path.dirname( os.path.abspath( os.path.dirname( __file__ ) ) ), 'common' )] + sys.path

import config
import tools
import yadsync

from PyQt4.QtGui import *
from PyQt4.QtCore import *
from PyKDE4.kdecore import *
from PyKDE4.kdeui import *
from PyKDE4.kio import *

from settingsdialog import *

_=gettext.gettext


class MainWindow( KMainWindow ):
	def __init__( self, config, kapp, kaboutdata ):
		KMainWindow.__init__( self )

		self.config = config
		self.kapp = kapp
		self.kaboutdata = kaboutdata

		main_widget = QWidget( self )
		main_widget.setMinimumWidth( 500 )
		self.setCentralWidget( main_widget )
		main_layout = QVBoxLayout( main_widget )

		#profiles
		layout = QHBoxLayout()
		main_layout.addLayout( layout )

		layout.addWidget( QLabel( QString.fromUtf8( _('Profile:') ), self ) )

		self.combo_profiles = KComboBox( self )
		layout.addWidget( self.combo_profiles, 1 )
		QObject.connect( self.combo_profiles, SIGNAL('currentIndexChanged(int)'), self.current_profile_changed )

		#add sync/save/restore
		layout = QVBoxLayout()
		main_layout.addLayout( layout )

		self.btn_sync = KPushButton( KIcon( 'view-refresh' ), QString.fromUtf8( _('Synchronize') ), self )
		self.btn_sync.setMinimumHeight( 50 )
		QObject.connect( self.btn_sync, SIGNAL('clicked()'), self.sync )
		layout.addWidget( self.btn_sync )

		self.btn_save = KPushButton( KIcon( 'arrow-right' ), QString.fromUtf8( _('Save') ), self )
		self.btn_save.setMinimumHeight( 50 )
		QObject.connect( self.btn_save, SIGNAL('clicked()'), self.save )
		layout.addWidget( self.btn_save )

		self.btn_restore = KPushButton( KIcon( 'arrow-left' ), QString.fromUtf8( _('Restore') ), self )
		self.btn_restore.setMinimumHeight( 50 )
		QObject.connect( self.btn_restore, SIGNAL('clicked()'), self.restore )
		layout.addWidget( self.btn_restore )

		#add settings/about/help/quit
		layout = QHBoxLayout()
		main_layout.addLayout( layout )

		btn = KPushButton( KStandardGuiItem.configure(), self )
		QObject.connect( btn, SIGNAL('clicked()'), self.show_settings )
		layout.addWidget( btn )

		btn = KPushButton( KIcon( 'help-about' ), QString.fromUtf8( _('About') ), self )
		QObject.connect( btn, SIGNAL('clicked()'), self.show_about )
		layout.addWidget( btn )

		btn = KPushButton( KStandardGuiItem.help(), self )
		QObject.connect( btn, SIGNAL('clicked()'), self.show_not_implemented )
		layout.addWidget( btn )

		btn = KPushButton( KStandardGuiItem.quit(), self )
		QObject.connect( btn, SIGNAL('clicked()'), self.close )
		layout.addWidget( btn )

		#add output log
		layout = QVBoxLayout()
		main_layout.addLayout( layout )

		layout.addWidget( QLabel( QString.fromUtf8( _('Output Log:') ) ) )

		self.txt_log = KRichTextWidget( self )
		self.txt_log.setReadOnly( True )
		layout.addWidget( self.txt_log )

		#link to yadsync
		layout = QHBoxLayout()
		main_layout.addLayout( layout )

		layout.addStretch()

		url_label = KUrlLabel( 'http://yadsync.le-web.org', 'yadsync.le-web.org', self )
		QObject.connect( url_label, SIGNAL('leftClickedUrl(const QString&)'), self.open_url );
		layout.addWidget( url_label )

		#
		self.update_profiles()

	def current_profile_changed( self, index ):
		profile_id = str( self.combo_profiles.itemData( index ).toString().toUtf8() )
		if len( profile_id ) <= 0:
			return
		
		if profile_id != self.config.get_current_profile():
			self.config.set_current_profile( profile_id )
			self.update_profile()

	def open_url( self, url ):
		self.run = KRun( KUrl( url ), self, True )

	def sync( self, update_src = True, update_dest = True ):
		self.setEnabled( False )
		self.log_clear()
		self._update_loop_()

		yadsync.YADSync( self.config, self._sync_callback_ ).sync( update_src, update_dest )

		self.setEnabled( True )

	def save( self ):
		self.sync( False, True )

	def restore( self ):
		self.sync( True, False )

	def update_profiles( self ):
		profiles = self.config.get_profiles_sorted_by_name()
		current_profile_id = self.config.get_current_profile()

		self.combo_profiles.clear()
			
		for profile_id in profiles:
			self.combo_profiles.addItem( QString.fromUtf8( self.config.get_profile_name( profile_id ) ), QVariant( QString.fromUtf8( profile_id ) ) )
			if profile_id == current_profile_id:
				self.combo_profiles.setCurrentIndex( self.combo_profiles.count() - 1 )

		self.update_profile()

	def update_profile( self ):
		enabled = self.config.can_sync()
		self.btn_sync.setEnabled( enabled )
		self.btn_save.setEnabled( enabled )
		self.btn_restore.setEnabled( enabled )

	def show_settings( self ):
		SettingsDialog( self ).exec_()
		self.update_profiles()

	def show_about( self ):
		KAboutApplicationDialog( self.kaboutdata, self ).exec_()

	def show_not_implemented( self ):
		KMessageBox.error( self, QString.fromUtf8( _('Not implemented') ) )

	def log_clear( self ):
		self.txt_log.setText( QString.fromUtf8( '' ) )
		self._update_loop_()

	def log_append( self, text, tag = None ):
		color = 'black';
		font_family = 'sans';
		font_weight = QFont.Normal

		if not tag is None:
			if tag == 'title':
				font_weight = QFont.Bold
			elif tag.startswith( 'cmd' ):
				font_family = 'monospace'

			if tag.endswith( 'warning' ):
				font_weight = QFont.Bold
				color = 'blue'
			elif tag.endswith( 'error' ):
				font_weight = QFont.Bold
				color = 'red'
			elif tag.endswith( 'ok' ):
				font_weight = QFont.Bold
	
		self.txt_log.setFontFamily( font_family )
		self.txt_log.setFontWeight( font_weight )
		self.txt_log.setTextColor( QColor( color ) )

		self.txt_log.append( QString.fromUtf8( text ) )

		self._update_loop_()

	def _update_loop_( self ):
		while self.kapp.hasPendingEvents():
			self.kapp.processEvents()

	def _sync_callback_( self, type, message ):
		self.log_append( message, type )

		if type in [ 'error', 'warning', 'title', 'cmd.ok', 'cmd.error', 'cmd.warning' ]:
			self.log_append( '' )


def create_kapplication( cfg ):
	kaboutdata = KAboutData( 'yadsync', '', ki18n( cfg.APP_NAME ), cfg.VERSION, ki18n( '' ), KAboutData.License_GPL_V2, ki18n( cfg.COPYRIGHT ), ki18n( '' ), 'http://yadsync.le-web.org', 'dan@le-web.org' )
	kaboutdata.setProgramIconName( 'view-refresh' )

	translation_text = cfg.get_translations()
	translators = []
	translator_emails = []
	for line in translation_text.split( '\n' ):
		line = line.strip()
		if len( line ) == 0:
			continue
		
		index1 = line.find( ':' )
		if index1 < 0:
			continue

		index2 = line.find( '<', index1 )
		if index2 < 0:
			continue

		index3 = line.find( '>', index2 )
		if index3 < 0:
			continue

		translators.append( line[ index1 + 1 : index2 ].strip() )
		translator_emails.append( line[ index2 + 1 : index3 ].strip() )

	kaboutdata.setTranslator( ki18n( ','.join( translators ) ), ki18n( ','.join( translator_emails ) ) )

	#KCmdLineArgs.init( sys.argv, kaboutdata )
	KCmdLineArgs.init( [sys.argv[0]], kaboutdata )
	return ( KApplication(), kaboutdata )


def run():
	cfg = yadsync.app( 'yadsync-kde4' )
	kapp, kaboutdata = create_kapplication( cfg )
	main_window = MainWindow( cfg, kapp, kaboutdata )
	main_window.show()
	kapp.exec_()


if __name__ == '__main__':
	run()

