#    YADSync
#    Copyright (C) 2009 Oprea Dan
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import os.path
import gettext
import copy

from PyQt4.QtGui import *
from PyQt4.QtCore import *
from PyKDE4.kdecore import *
from PyKDE4.kdeui import *
from PyKDE4.kio import *

import config
import tools


_=gettext.gettext


class SettingsDialog( KDialog ):
	def __init__( self, parent ):
		KDialog.__init__( self, parent )

		self.config = parent.config
		self.config_copy_dict = copy.copy( self.config.dict )
		self.current_profile_org = self.config.get_current_profile()

		self.setWindowIcon( KIcon( 'configure' ) )
		self.setCaption( QString.fromUtf8( _( 'Settings' ) ) )

		main_widget = QWidget( self )
		main_widget.setMinimumWidth( 600 )
		main_layout = QVBoxLayout( main_widget )
		self.setMainWidget( main_widget )

		#profiles
		layout = QHBoxLayout()
		main_layout.addLayout( layout )

		layout.addWidget( QLabel( QString.fromUtf8( _('Profile:') ), self ) )

		self.combo_profiles = KComboBox( self )
		layout.addWidget( self.combo_profiles, 1 )
		QObject.connect( self.combo_profiles, SIGNAL('currentIndexChanged(int)'), self.current_profile_changed )

		btn = KPushButton( KStandardGuiItem.properties(), self )
		self.btn_profiles_edit = btn
		QObject.connect( btn, SIGNAL('clicked()'), self.edit_profile )
		layout.addWidget( btn )

		btn = KPushButton( KStandardGuiItem.add(), self )
		self.btn_profiles_add = btn
		QObject.connect( btn, SIGNAL('clicked()'), self.add_profile )
		layout.addWidget( btn )

		btn = KPushButton( KStandardGuiItem.del_(), self )
		self.btn_profiles_remove = btn
		QObject.connect( btn, SIGNAL('clicked()'), self.remove_profile )
		layout.addWidget( btn )

		#tabs
		tab_widget = KTabWidget( self )
		main_layout.addWidget( tab_widget )
	
		#TAB: Folders
		tab = QWidget( self )
		tab_widget.addTab( tab, QString.fromUtf8( _( 'Folders' ) ) )
		layout = QVBoxLayout( tab )

		#list_folders
		self.list_folders = QTreeWidget( self )
		layout.addWidget( self.list_folders )
		self.list_folders.setRootIsDecorated( False )
		self.list_folders.setEditTriggers( QAbstractItemView.NoEditTriggers )
		self.list_folders.setHeaderLabels( [ QString.fromUtf8( _('Source') ), QString.fromUtf8( _('Destination') ) ] )
		self.list_folders.header().setResizeMode( 0, QHeaderView.Stretch )

		grid_layout = QGridLayout()
		layout.addLayout( grid_layout )

		#source
		grid_layout.addWidget( QLabel( QString.fromUtf8( _('Source:') ) ), 0, 0 )

		self.edit_src = KLineEdit( self )
		grid_layout.addWidget( self.edit_src, 0, 1 )
		self.edit_src.setReadOnly( True )

		btn = KPushButton( KIcon( 'folder' ), '', self )
		QObject.connect( btn, SIGNAL('clicked()'), self.select_source )
		grid_layout.addWidget( btn, 0, 2 )

		#destination
		grid_layout.addWidget( QLabel( QString.fromUtf8( _('Destination:') ) ), 1, 0 )

		self.edit_dest = KLineEdit( self )
		grid_layout.addWidget( self.edit_dest, 1, 1 )
		self.edit_dest.setReadOnly( True )

		btn = KPushButton( KIcon( 'folder' ), '', self )
		QObject.connect( btn, SIGNAL('clicked()'), self.select_dest )
		grid_layout.addWidget( btn, 1, 2 )

		#add/remove buttons
		buttons_layout = QHBoxLayout()
		layout.addLayout( buttons_layout )

		buttons_layout.addStretch()

		btn = KPushButton( KStandardGuiItem.add(), self )
		QObject.connect( btn, SIGNAL('clicked()'), self.add_src_dest )
		buttons_layout.addWidget( btn )

		btn = KPushButton( KStandardGuiItem.del_(), self )
		QObject.connect( btn, SIGNAL('clicked()'), self.remove_src_dest )
		buttons_layout.addWidget( btn )
		self.btn_remove_src_dest = btn

		#TAB: Exclude
		tab = QWidget( self )
		tab_widget.addTab( tab, QString.fromUtf8( _( 'Exclude' ) ) )
		layout = QVBoxLayout( tab )

		self.list_exclude = QTreeWidget( self )
		layout.addWidget( self.list_exclude )
		self.list_exclude.setRootIsDecorated( False )
		self.list_exclude.setEditTriggers( QAbstractItemView.NoEditTriggers )
		self.list_exclude.setHeaderLabels( [ QString.fromUtf8( _('Exclude') ) ] )
		self.list_exclude.header().setResizeMode( 0, QHeaderView.Stretch )

		#add/remove buttons
		buttons_layout = QHBoxLayout()
		layout.addLayout( buttons_layout )

		buttons_layout.addStretch()

		btn = KPushButton( KStandardGuiItem.add(), self )
		QObject.connect( btn, SIGNAL('clicked()'), self.add_new_exclude )
		buttons_layout.addWidget( btn )

		btn = KPushButton( KStandardGuiItem.del_(), self )
		buttons_layout.addWidget( btn )
		QObject.connect( btn, SIGNAL('clicked()'), self.remove_exclude )
		self.btn_remove_exclude = btn

		#TAB: Options
		tab = QWidget( self )
		tab_widget.addTab( tab, QString.fromUtf8( _( 'Options' ) ) )
		layout = QVBoxLayout( tab )

		self.cb_enable_backup = QCheckBox( QString.fromUtf8( _( 'Enable backup (\'.backup\' sub-folder)' ) ), self )
		layout.addWidget( self.cb_enable_backup )

		self.cb_enable_delete = QCheckBox( QString.fromUtf8( _( 'Enable delete for save/restore' ) ), self )
		layout.addWidget( self.cb_enable_delete )

		layout.addStretch()

		#
		self.update_profiles()

	def current_profile_changed( self, index ):
		profile_id = str( self.combo_profiles.itemData( index ).toString().toUtf8() )
		if len( profile_id ) <= 0:
			return
		
		if profile_id != self.config.get_current_profile():
			self.save_current_profile()
			self.config.set_current_profile( profile_id )
			self.update_profile()

	def edit_profile( self ):
		ret_val = KInputDialog.getText( QString.fromUtf8( _( 'Rename profile' ) ), QString.fromUtf8( _( 'Name:' ) ), '', self )
		if not ret_val[1]:
			return
		
		name = str( ret_val[0].toUtf8() ).strip()
		if len( name ) <= 0:
			return

		if not self.config.set_profile_name( name ):
			return

		self.update_profiles()

	def add_profile( self ):
		ret_val = KInputDialog.getText( QString.fromUtf8( _( 'New profile' ) ), QString.fromUtf8( _( 'Name:' ) ), '', self )
		if not ret_val[1]:
			return
		
		name = str( ret_val[0].toUtf8() ).strip()
		if len( name ) <= 0:
			return

		self.save_current_profile()

		if not self.config.add_profile( name ):
			return

		self.update_profiles()

	def remove_profile( self ):
		if self.question_handler( _('Are you sure you want to delete the profile "%s" ?') % self.config.get_profile_name() ):
			self.config.remove_profile()
			self.update_profiles()

	def update_profiles( self ):
		self.update_profile()
		current_profile_id = self.config.get_current_profile()

		self.combo_profiles.clear()
			
		profiles = self.config.get_profiles_sorted_by_name()
		index = 0

		for profile_id in profiles:
			self.combo_profiles.addItem( QString.fromUtf8( self.config.get_profile_name( profile_id ) ), QVariant( QString.fromUtf8( profile_id ) ) )
			if profile_id == current_profile_id:
				self.combo_profiles.setCurrentIndex( self.combo_profiles.count() - 1 )

	def update_profile( self ):
		if self.config.get_current_profile() == '1':
			self.btn_profiles_edit.setEnabled( False )
			self.btn_profiles_remove.setEnabled( False )
		else:
			self.btn_profiles_edit.setEnabled( True )
			self.btn_profiles_remove.setEnabled( True )

		#populate folders
		self.list_folders.clear()
		self.btn_remove_src_dest.setEnabled( False )

		for item in self.config.get_directories():
			self.add_folder( item[0], item[1] )

		#populate exclude
		self.list_exclude.clear()
		self.btn_remove_exclude.setEnabled( False )

		for item in self.config.get_exclude():
			self.add_exclude( item )

		#options
		self.cb_enable_backup.setChecked( self.config.get_enable_backup() )
		self.cb_enable_delete.setChecked( self.config.get_enable_delete() )

	def error_handler( self, message ):
		KMessageBox.error( self, QString.fromUtf8( message ) )

	def question_handler( self, message ):
		return KMessageBox.Yes == KMessageBox.warningYesNo( self, QString.fromUtf8( message ) )

	def select_source( self ):
		path = str( KFileDialog.getExistingDirectory( KUrl(), self, QString.fromUtf8( _( 'Source folder' ) ) ).toUtf8() )
		if len( path ) == 0 :
			return

		path = tools.prepare_path( path )
		self.edit_src.setText( QString.fromUtf8( path ) )

	def select_dest( self ):
		path = str( KFileDialog.getExistingDirectory( KUrl(), self, QString.fromUtf8( _( 'Destination folder' ) ) ).toUtf8() )
		if len( path ) == 0 :
			return

		path = tools.prepare_path( path )
		self.edit_dest.setText( QString.fromUtf8( path ) )

	def add_folder( self, source, dest ):
		item = QTreeWidgetItem()
		item.setIcon( 0, KIcon( 'folder' ) )
		item.setText( 0, QString.fromUtf8( source ) )
		item.setIcon( 1, KIcon( 'arrow-right' ) )
		item.setText( 1, QString.fromUtf8( dest ) )
		self.list_folders.addTopLevelItem( item )

		if self.list_folders.currentItem() is None:
			self.list_folders.setCurrentItem( item )

		self.btn_remove_src_dest.setEnabled( True )

		return item

	def add_exclude( self, exclude ):
		item = QTreeWidgetItem()
		item.setIcon( 0, KIcon( 'dialog-cancel' ) )
		item.setText( 0, QString.fromUtf8( exclude ) )
		self.list_exclude.addTopLevelItem( item )

		if self.list_exclude.currentItem() is None:
			self.list_exclude.setCurrentItem( item )

		self.btn_remove_exclude.setEnabled( True )

		return item

	def remove_exclude( self ):
		item = self.list_exclude.currentItem()

		if not item is None:
			index = self.list_exclude.indexOfTopLevelItem( item )
			if index >= 0:
				self.list_exclude.takeTopLevelItem( index )

				if self.list_exclude.topLevelItemCount() > 0:
					self.list_exclude.setCurrentItem( self.list_exclude.topLevelItem(0) )

		if self.list_exclude.topLevelItemCount() <= 0:
			self.btn_remove_exclude.setEnabled( False )

	def add_new_exclude( self ):
		ret_val = KInputDialog.getText( QString.fromUtf8( _( 'Exclude pattern' ) ), '', '', self )
		if not ret_val[1]:
			return
		
		pattern = str( ret_val[0].toUtf8() ).strip()
		if len( pattern ) <= 0:
			KMessageBox.error( self, QString.fromUtf8( _('Exclude pattern can\'t be empty') ) )
			return

		for index in xrange( self.list_folders.topLevelItemCount() ):
			item = self.list_folders.topLevelItem( index )
			item_pattern = str( item.text(0).toUtf8() )

			if item_pattern == pattern:
				KMessageBox.error( self, QString.fromUtf8( _('Pattern "%s" already exists !') % pattern ) )
				return

		self.add_exclude( pattern )

	def remove_src_dest( self ):
		item = self.list_folders.currentItem()

		if not item is None:
			index = self.list_folders.indexOfTopLevelItem( item )
			if index >= 0:
				self.list_folders.takeTopLevelItem( index )

				if self.list_folders.topLevelItemCount() > 0:
					self.list_folders.setCurrentItem( self.list_folders.topLevelItem(0) )

		if self.list_folders.topLevelItemCount() <= 0:
			self.btn_remove_src_dest.setEnabled( False )

	def add_src_dest( self ):
		src_path = str( self.edit_src.text().trimmed().toUtf8() )
		if len( src_path ) <= 0:
			self.select_source()
			src_path = str( self.edit_src.text().trimmed().toUtf8() )
			if len( src_path ) <= 0:
				return

		dest_path = str( self.edit_dest.text().trimmed().toUtf8() )
		if len( dest_path ) <= 0:
			self.select_dest()
			dest_path = str( self.edit_dest.text().trimmed().toUtf8() )
			if len( dest_path ) <= 0:
				return

		if src_path == dest_path:
			KMessageBox.error( self, QString.fromUtf8( _('Source and destination are the same !') ) )
			return

		if len( src_path ) <= 1:
			KMessageBox.error( self, QString.fromUtf8( _('Source can\'t be root !') ) )
			return

		if tools.is_local_path( src_path ):
			if not os.path.isdir( src_path ):
				KMessageBox.error( self, QString.fromUtf8( _('Source folder don\'t exists !') ) )
				return

		if len( dest_path ) <= 1:
			KMessageBox.error( self, QString.fromUtf8( _('Destination can\'t be root !') ) )
			return

		if tools.is_local_path( dest_path ):
			if not os.path.isdir( dest_path ):
				KMessageBox.error( self, QString.fromUtf8( _('Destination folder don\'t exists !') ) )
				return

		for index in xrange( self.list_folders.topLevelItemCount() ):
			item = self.list_folders.topLevelItem( index )
			item_src = str( item.text(0).toUtf8() )
			item_dest = str( item.text(1).toUtf8() )

			if item_src == src_path and item_dest == dest_path:
				KMessageBox.error( self, QString.fromUtf8( _('Source and destination already exists !') ) )
				return

			if item_src == src_path:
				if KMessageBox.Yes != KMessageBox.warningYesNo( self, QString.fromUtf8( _('Source folder already exists\n%s\nAre you sure you want to add it again ?') % src_path ) ):
					return

			if item_dest == dest_path:
				if KMessageBox.Yes != KMessageBox.warningYesNo( self, QString.fromUtf8( _('Destination folder already exists:\n%s\nAre you sure you want to add it again ?') % dest_path ) ):
					return

		#add source <=> destination
		self.add_folder( src_path, dest_path )

		self.edit_src.setText('')
		self.edit_dest.setText('')

	def exec_( self ):
		self.config.set_question_handler( self.question_handler )
		self.config.set_error_handler( self.error_handler )
		ret_val = KDialog.exec_( self )
		self.config.clear_handlers()

		if ret_val != QDialog.Accepted:
			self.config.dict = self.config_copy_dict
			
		self.config.set_current_profile( self.current_profile_org )

		return ret_val

	def save_current_profile( self ):
		#directories
		items = []

		for index in xrange( self.list_folders.topLevelItemCount() ):
			item = self.list_folders.topLevelItem( index )
			item_src = str( item.text(0).toUtf8() )
			item_dest = str( item.text(1).toUtf8() )
			items.append( [ item_src, item_dest ] )

		self.config.set_directories( items )

		#exclude
		items = []

		for index in xrange( self.list_exclude.topLevelItemCount() ):
			item = self.list_exclude.topLevelItem( index )
			item_pattern = str( item.text(0).toUtf8() )
			items.append( item_pattern )

		self.config.set_exclude( items )

		#options
		self.config.set_enable_backup( self.cb_enable_backup.isChecked() )
		self.config.set_enable_delete( self.cb_enable_delete.isChecked() )

	def accept( self ):
		self.save_current_profile()

		#check
		profiles = self.config.get_profiles()
		for profile_id in profiles:
			if len( self.config.get_directories(profile_id) ) <= 0:
				KMessageBox.error( self, QString.fromUtf8( _('Profile: %s') % self.config.get_profile_name( profile_id ) + '\n' + _('There are no folders !') ) )
				return False

		#save
		self.config.save()

		KDialog.accept( self )

